# -*- coding: UTF-8 -*-
import math
import re
import os
import sys
import json

if sys.version_info[0]==2:
    from codecs import open

class MapInHtml:
    def __init__(self, filename, **args):
        #args may be
        #center
        #zoom
        #if 'my_template' in args:
        #    raise Exception("Please change 'my_template' to 'template' arg name")
        self.result_f = None #нужно, чтобы если следующая строка вылетит с Exception в __del__ всё же был self.result_f
        
        templates_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "templates")
        template_filename = (args['my_template'] if 'my_template' in args
                        else os.path.join(templates_dir, args['template']) if 'template' in args
                        else os.path.join(templates_dir, 'general.html'))
            
        self.template_f = open(template_filename, encoding="utf-8-sig")
        self.result_f = open(filename, "w", encoding="utf-8-sig")
        
        for line in self.template_f:
            self.result_f.write(line)
            if(re.match("//startRewriteFromHere", line)):
                break
        else:
            raise Exception("Looks like a problem in template file")
      
        center = args.get('center')
        zoom = args.get('zoom', 12)
        self.autozoom = args.get('autozoom')
        self.maxzoom = args.get("maxzoom")
        self.minzoom = args.get("minzoom")
        
        if center:
            self.write(self.get_map_header(center, zoom))
            self.center_found = True
        else:
            self.center_found = False
            self.zoom = zoom
            
        self.default_shortcuts = {
            "color": "fillColor",
            "opacity": "fillOpacity",
            "text": "balloonContent",
            "width": "strokeWidth"
        }
    
    def close(self):
        if not self.result_f:
            return
        
        skipping = True
        for line in self.template_f:
            if skipping:
                if re.match("//endRewriteHere", line):
                    if self.autozoom:
                        self.write("myMap.setBounds(myMap.geoObjects.getBounds());\n")
                    skipping = False
                else:
                    continue
            self.write(line)
        if skipping:
            raise Exception("Looks like a problem in template file")
            
        self.result_f.close()
        self.template_f.close()
        self.result_f = None

    def __enter__(self):
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def __del__(self):
        self.close()

    def find_center(self, coords):
        if self.center_found:
            return
        self.write(self.get_map_header(coords[0], self.zoom))
        self.center_found = True
    
    def write(self, code):
        self.result_f.write(code)
       
    def get_map_header(self, center_coords, zoom):
        second_params = ""
        if self.minzoom:
            second_params += "minZoom: %d, "%self.minzoom
        if self.maxzoom:
            second_params += "maxZoom: %d, "%self.maxzoom
        if second_params:
            second_params = ", {%s}" % second_params
        return ('var myMap = new ymaps.Map("map", '+
                '{center: [%f, %f], zoom: %d}%s);\n'%(
                center_coords[0], center_coords[1], zoom, second_params))
    
    def text_for_js(self, t, block_html=False):
        if sys.version_info[0]>=3:
            t = str(t)
        else:
            t = unicode(t)
        if block_html:
            t = re.sub("<", "&lt;", re.sub(">", "&gt;", t))
        return json.dumps(t, ensure_ascii=False)

    def params_parser(self, params, **shortcuts):
        param1 = param2 = ""

        if 'text' in params:
            what = shortcuts.get("text", self.default_shortcuts["text"])
            param1 += '%s: %s, '%(what, self.text_for_js(params['text']))
        if 'balloon_content' in params:
            param1 += 'balloonContent: %s, '%self.text_for_js(params['balloon_content'])
        if 'hint_content' in params:
            param1 += 'hintContent: %s, '%self.text_for_js(params['hint_content'])
        if 'icon_content' in params:
            param1 += 'iconContent: %s, '%self.text_for_js(params['icon_content'])
        if 'icon_caption' in params:
            param1 += 'iconCaption: %s, '%self.text_for_js(params['icon_caption'])

        if 'color' in params:
            what = shortcuts.get("color", self.default_shortcuts["color"]) 
            param2 += '%s: %s, '%(what, self.text_for_js(params['color']))
        if 'opacity' in params:
            what = shortcuts.get("opacity", self.default_shortcuts["opacity"]) 
            param2 += '%s: %s, '%(what, self.text_for_js(params['opacity']))
        if 'width' in params:
            what = shortcuts.get("width", self.default_shortcuts["width"]) 
            param2 += '%s: %.0f, '%(what, params['width'])
        if 'stroke_width' in params:
            param2 += 'strokeWidth: %.0f, '%params['stroke_width']
        if 'stroke_opacity' in params:
            param2 += 'strokeOpacity: %.3f, '%params['stroke_opacity']            
        if 'stroke_color' in params:
            param2 += 'strokeColor: %s, '%self.text_for_js(params['stroke_color'])
        if 'fill_opacity' in params:
            param2 += 'fillOpacity: %.3f, '%params['fill_opacity']            
        if 'fill_color' in params:
            param2 += 'fillColor: %s, '%self.text_for_js(params['fill_color'])
            
        #print("adfasdfa", param1, param2)    
        return (param1, param2)
    
    
    def placemark(self, coord, **params):
        self.find_center([coord])
        param1, param2 = self.params_parser(params, color="iconColor")
        if 'preset' in params:
            param2 += "preset: %s, "%self.text_for_js(params['preset'])
        if 'icon_caption_max_width' in params:
            param2 += "iconCaptionMaxWidth: %s, "%self.text_for_js(params['icon_caption_max_width'])
            
        self.write('myMap.geoObjects.add(new ymaps.Placemark([%f, %f],'%(coord[0], coord[1]) +
                   '{%s}, {%s}));\n'%(param1, param2))

    def circle(self, coord, radius, **params):
        self.find_center([coord])
        if 'stroke_width' not in params:
            params['stroke_width'] = 0
        param1, param2 = self.params_parser(params, color="fillColor", opacity="fillOpacity")
            
        self.write('myMap.geoObjects.add(new ymaps.Circle([[%f, %f], %f],'%(coord[0], coord[1], radius) +
                   '{%s}, {%s}));\n'%(param1, param2))

    def polyline(self, coords, **params):
        self.find_center(coords)
        param1, param2 = self.params_parser(params, color="strokeColor", opacity="strokeOpacity")
        self.write('myMap.geoObjects.add(new ymaps.Polyline([%s], {%s}, {%s}));\n'%
                                 (",".join("[%f,%f]"%(c[0], c[1]) for c in coords),
                                  param1, param2))

    def rectangle(self, coords, **params):
        self.find_center(coords)
        param1, param2 = self.params_parser(params, color="fillColor", opacity="fillOpacity")
        self.write('myMap.geoObjects.add(new ymaps.Rectangle([%s], {%s}, {%s}));\n'%
                                 (",".join("[%f,%f]"%(c[0], c[1]) for c in coords),
                                  param1, param2))


if __name__ == '__main__':
    with MapInHtml("draw_on_map_test.html", zoom=9, autozoom=True, minzoom=5, maxzoom=15) as draw:
        draw.placemark([57, 35.5], text="test", color="#FF0000")
        draw.circle([57.5, 35], 3000, text=u"русский test circle", color="#0000FF40")
        draw.polyline([[57, 35], [57.5, 35], [57, 35.5]], color="#C080C0", opacity=0.5, width=3)
        draw.rectangle([[57.5, 35], [57.6, 35.1]], width=0)
    
        s = 'Full list of icon style presets <A href="https://tech.yandex.ru/maps/doc/jsapi/2.1/ref/reference/option.presetStorage-docpage" target="blank">here</A>'
        draw.placemark([57, 35], text=s, preset="islands#greenDotIconWithCaption",
                         icon_caption="Click me", icon_caption_max_width = 100)

#     draw.close()
    